# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

import sys
import os
import locator

"""""
Single file / folder / (later list.txt / regex): -> also generates link file!
10f [-i][-f][-r][-n] name foldername filename

-i = insert/update data (as opposed to download)
-f = folder
-u = update
-n = no error if linkfile already exists

# Get data, ONLY use the link file
10f name.10f

"""

def print_specific_help(specific_help_type):
    if specific_help_type == '-i':
        print('Insert: use to insert new data:')
        print('example:')
        print('10f -i dataname foldername filename')
        print('10f -i -f dataname foldername [regex]')
    if specific_help_type == '-t':
        print('Download data from a link.')
        print('example:')
        print('10f -t mylink.10f')
        print('10f -t mylink.10f tmp/storage/personal')
        print('Download without any parameters does not need the -t')
        print('10f mylink.10f')
    if specific_help_type == '-f':
        print('Folder: Inserts the folder as data.')
    if specific_help_type == '-u':
        print('Update / Refresh (use this if the data already exists).')
        print('example:')
        print('10f -u dataname foldername filename')
        print('10f -u -f dataname foldername  [regex]')
    if specific_help_type == '-n':
        print('Overwrites without error if the resulting linkfile exists.')
    if specific_help_type == '-c':
        print('Create a link')
        print('example:')
        print('10f -c dataname')
    if specific_help_type == '-l':
        print('List information')
        print('example for the global database:')
        print('example for the global database:')
        print('10f -l')
        print('example for a specific entry:')
        print('10f -l dataname')
    if specific_help_type == '-s':
        print('Set wanted number of fingers')
        print('A bigger number equals your data will be (tried to be) shared more')
        print('which makes your computer likely to share more data too.')
        print('A smaller number, less.')
        print('example:')
        print('10f -s dataname 10')
    if specific_help_type == '-d':
        print('Delete one of your datas or just their fingers')
        print('example:')
        print('Delete data and fingers:')
        print('10f -d dataname')
        print('Delete only fingers:')
        print('10f -d -f dataname')
    if specific_help_type == '-s':
        print('Sets the number of fingers we would like for this data')
        print('Note: each successful finger is a node that shares your data')
        print('For each finger you share their data as they share yours.')
        print('Default is 10')
        print('example:')
        print('10f -s dataname 20')

    sys.exit(0)

def print_help():
    print('Tenfingers usage:')
    print('')
    print('10f [-i][-f][-u][-n][-l] name [foldername] [filename][regex]')
    print('10f -t linkfile [destfolder]')
    print('')
    print('-i [-f] = insert/update data (as opposed to download)')
    print('-u [-f] = update/refresh')
    print('-f = use a folder, not a file')
    print('-c = create a link')
    print('-n = no error if linkfile already exists')
    print('-l [name] = list information')
    print('-s name number_of_fingers_wanted = set how many times you want to exchange shares for this data')
    print('-d [-f] name = delete your shared data')
    print('-h [-i,-t,-f,-r,-n,-c,-s,-d] specific help')
    print('')
    print('Download data with a link, say link.10f')
    print('./10f link.10f')
    sys.exit(0)


if len(sys.argv) == 1:
    print_help()


cmd = {}
commands = {'-h', '-t', '-i', '-f', '-u', '-n', '-c', '-l', '-s', '-d'}
specific_help = False
specific_help_type = False

for c in sys.argv:
    if c[:1] == '-':
        if specific_help and not specific_help_type:
            specific_help_type = c

        if c == '-h':  # Specific help:
            specific_help = True
        if c in commands:
            cmd[c] = True
        else:
            print('Unknown command:', c)
            print_help()

if specific_help:
    if specific_help_type == False:
        print_help()
    else:
        print_specific_help(specific_help_type)
        sys.exit(-1)

noerror = '-n' in cmd

if '-i' in cmd:
    from insert_data import main as insert_data_main
    if '-f' in cmd: # variable length number of argv, regex is optional
        print('Insert folder:')
        if len(sys.argv) == 4:
            insert_data_main(['', sys.argv[2], sys.argv[3]])  # To test
        if len(sys.argv) == 5:
            insert_data_main(['', sys.argv[2], sys.argv[3], sys.argv[4]])  # To test
    else:
        print('Insert file:')
        insert_data_main(['', sys.argv[2], sys.argv[3], sys.argv[4]])  # WORKS

    # make the link NO because it's stupid
    #if len(sys.argv) >= 3:
    #    from get_link_from_data import create_and_save_link
    #    create_and_save_link(sys.argv[2])

    sys.exit(0)

if '-c' in cmd:
    if len(sys.argv) == 3:
        print('Create link for', sys.argv[2])
        from get_link_from_data import create_and_save_link_database
        create_and_save_link_database(sys.argv[2])
    else:
        print_specific_help('-c')

    sys.exit(0)

if '-t' in cmd:
    if len(sys.argv) == 3 or len(sys.argv) == 4:
        destfolder = './'
        if len(sys.argv) == 4:
            destfolder = sys.argv[3]
        # def get_data(linkfile, dest_folder):
        from get_data_from_link import get_data
        get_data(sys.argv[2], destfolder)
        sys.exit(0)

if '-l' in cmd:
    # List stuff
    import database_query as dq
    from helpers import human_readable_file_size

    if len(sys.argv) == 3:
        print('Show information for the data ['+sys.argv[2]+']')
        # Show the fingers for this entry:
        dq.flush_fingers(sys.argv[2])
    else:
        print('Show general information:')
        # Flush generic data about nodes, data etc.
        dq.flush_owned_data()
        dq.flush_shared_data()

    our_data_size = dq.get_owned_files_size()
    shared_data_size = dq.get_shared_files_size()
    print('Owned data: ', human_readable_file_size(our_data_size))
    print('Shared data:', human_readable_file_size(shared_data_size))

    sys.exit(0)

if '-s' in cmd:  # 10f -s name nof_fingers
    if len(sys.argv) == 4:
        nof_fingers = int(sys.argv[3])
        if nof_fingers is not None and nof_fingers > 0:
            print('Update number of fingers is not yet implemented')
        else:
            print_specific_help('-f')
        sys.exit(0)


if '-d' in cmd:  # 10f -d [-f] name
    name = None
    delete_fingers_only = False
    if len(sys.argv) == 3:
        name = sys.argv[2]
    if len(sys.argv) == 4 and sys.argv[2] == '-f':
        name = sys.argv[3]
        delete_fingers_only = True
    if name is not None:
        import database_query as dq
        if dq.delete_fingers(name):
            print('Fingers deleted for:', name)
        else:
            print('The data [' + str(name) + '] does not exist.')
        if not delete_fingers_only:
            if dq.delete_owned_data(name):
                owned_file_name = os.path.join(locator.module_path(), 'data/local/', (name + '.aes'))
                os.remove(owned_file_name)
                print('Deletion of owned data [' + str(name) + '] successful.')
            else:
                print('The owned data [' + str(name) + '] does not exist.')
        sys.exit(0)
    else:
        print('huh?')



if '-u' in cmd:
    from update_data import main as update_data_main
    if '-f' in cmd:
        print('Update folder:')
        if len(sys.argv) == 4:
            update_data_main(['', sys.argv[2], sys.argv[3]])  # To test
        if len(sys.argv) == 5:
            update_data_main(['', sys.argv[2], sys.argv[3], sys.argv[4]])  # To test
    else:
        print('Update file:')
        update_data_main(['', sys.argv[2], sys.argv[3], sys.argv[4]])  # To test

    sys.exit(0)

#from get_link_from_data import create_and_save_link
#create_and_save_link(linkfile)  # to test


if len(cmd) == 0:
    if len(sys.argv) >= 2:
        linkfile = sys.argv[1]
        print('Load:', linkfile)
        from get_data_from_link import get_data_from_database_link

        destfolder = './'
        if len(sys.argv) == 3:
            destfolder = sys.argv

        get_data_from_database_link(linkfile, destfolder)

        sys.exit(0)





print_help()

































