# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

import sys
import os

from Crypto.Cipher import AES
from Crypto.Util import Counter
from Crypto import Random
import binascii
import os.path
import tarfile

import database_connection as dc
import database_query as dq

from encryption import CipherHelper
import locator


# If called directly (e.g python thisfile.py then a check on __main__ at the end of this file will call main here, with any command line args.)
# from thisfile import main as this_main
# this_main(args)   # <- imports and calls the func here from another python script
def main(argv):
    print('INSERT DATA', len(argv))
    if len(argv) == 1:
        print('Usage:')
        print('To share a file:')
        print('python3 insert_data.py dataname filefolder file')
        print('To share a whole folder:')
        print('python3 insert_data.py -f dataname filefolder')
        # print('python3 insert_data.py --testrun')
        print('')
        print('Example:')
        print('python3 insert_data.py flower home/images flower.jpg')
        print('python3 insert_data.py -f my_website html/websites/my_site')

        sys.exit()

    """
    Generates one database entry from a specific file
    Database:
    NAME = The link-name, can be anything. This name will be used for the retrieval of the data.
    ORIGINALFILENAME = the files name when it will be generated from a link, without the path as the path is local.
    KEY, IV = AES keypair
    PUBKEY = an RSA public key (ours if we are publishing links, from other node if we have done a swicharoo with them and store their data)
    
    if NAME already exists, an update is done, using the already existing AES keypair.
    """

    # This loads and encrypts data, inserts it into the table 'data_owned'
    # stores encrypted data in folder '/data/local/' as it is our owned data.
    # Later on, the scheduler will ask other nodes for swicharoo (e.g storing our data at their place, and their data at our place in /data/$nodenr$/)
    def create_our_data(name, filefolder, file, rsa_pub_key):
        # Insert in database
        print('Insert new data:')
        print('Name:', name)
        print('Folder:', filefolder)
        print('File:', file)

        filefolder = filefolder.replace('\\', '/')
        file = file.replace('\\', '/')

        connection = dc.get_connection()

        filepath = os.path.join(filefolder, file)

        data = None
        with open(filepath, 'rb') as f:
            data = f.read()

        if data is None:
            print('File not found')
            sys.exit(-3)

        cursor = connection.cursor()
        cursor.execute("SELECT KEY, IV FROM data_owned WHERE NAME = '" + name + "'")
        rows = cursor.fetchall()
        # Does a database entry exists?
        if len(rows) > 0:
            print("#Error entry already exist (insert_data) use update_data.py instead")
            sys.exit(0)

        ch = CipherHelper()

        # Encrypt with AES-CTR
        encrypted_data = ch.encrypt(data)

        # Make a 'local' folder if it doesn't already exist
        directory = os.path.join(locator.module_path(), 'data/local/')
        if not os.path.exists(directory):
            os.makedirs(directory)

        # Store the encypted data in the data folder so listener can serve it to clients:
        with open(directory + name + '.aes', 'wb') as f:
            f.write(encrypted_data)

        AEShex = ch.aes_key.hex()
        IVhex = ch.raw_iv.hex()

        cursor.execute("INSERT INTO data_owned(NAME, ORIGINALFILENAME, KEY, IV, PUBKEY) VALUES(?, ?, ?, ?, ?);", (name, file, AEShex, IVhex, rsa_pub_key))
        connection.commit()


    # Test case:
    def test():
        print('[insert_data.py]')
        prv, pub = dq.get_rsa_hex_keys()
        create_our_data('test', 'my_local_data', 'test.txt', pub)
        create_our_data('image', 'my_local_data', 'test.png', pub)



    if argv[1] == '--testrun':
        test()
    else:
        if len(argv) >= 4:  # Use this to insert a link in the database with the local users rsa_pub_key
            # fetch the users rsa_pub_key
            prv, pub = dq.get_rsa_hex_keys()

            if argv[1] == '-f':
                print('Insert folder')
                regex = '.*'
                if len(argv) == 5:
                    regex = argv[4]
                folder = argv[3]
                # get all the files in 'folder' and zip it:
                data_name = argv[2]
                full_data_name = data_name+'.tar.gz'
                with tarfile.open(full_data_name, "w:gz") as tar_handle:
                    tar_handle.add(folder)

                print('Insert new folder link')
                # name, filefolder, file, rsa_pub_key
                create_our_data(data_name, './', full_data_name, pub)  # Local owned link
                # delete tar.gz file
                os.remove(full_data_name)
            else:
                print('Insert new link')
                # name, filefolder, file, rsa_pub_key
                create_our_data(argv[1], argv[2], argv[3], pub)  # Local owned link


    
    
# Call the main function with the command line arguments
if __name__ == '__main__':
    main(sys.argv)

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    