# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

# Run this to setup.
# Will not harm existing installations
# Generates RSA keypair, database etc.

import sys, os
import rsa_helper as rh
import generate_database as gdb
import database_connection as dc
import database_query as dq


def setup():
    print('[setup.py]')
    print('Welcome to the 1-2-10 finger setup!')

    # gen database?
    if not dc.database_exists():
        print('setup: generate database')
        gdb.create_software_database()

    # RSA keys exist?
    r = rh.get_rsa_keypair(True)
    if r is None:
        print('setup: generate RSA keys')
        # Generate them:
        prv, pub = rh.generate_rsa_keypair(4096)
        # insert in database
        dq.set_rsa(prv, pub)
        # Save off copy (this can be used for test systems, or for commanding our files from a distance)
        textkey_prv = prv.exportKey('DER').hex()
        with open('secret_key', 'w') as f:
            f.write(textkey_prv)
        textkey_pub = pub.exportKey('DER').hex()
        with open('public_key', 'w') as f:
            f.write(textkey_pub)

    else:
        print('setup: load RSA keys')
        prv, pub = r

    config_file_name = './config.json'
    if not os.path.isfile(config_file_name):
        print('Generating a configuration file (' + config_file_name + ') :')
        print('IMPORTANT NOTICE!')
        print('You need to configure this file with your public IP address')
        print('and chose a port for tenfingers!')
        print('This IP:PORT combo must be routed to tenfingers from outside.')
        with open(config_file_name, 'w') as f:
            f.write('{\n"ip": "127.0.0.1",\n"port": 16384,\n"unsecure_nick_name": "Alice",\n"LISTEN_READ_BYTES": 10000,\n"MAX_SIMULTANEOUS_CONNECTIONS": 5,\n"LISTEN_TIMEOUT_SECONDS": 5,\n"SERVER_SEND_PACKAGE_SIZE": 32768,\n"MAX_STORAGE_SPACE_OTHER_MB": 1024,\n"SWICHAROO_POLICY": "MAX_DOUBLE",\n"Policies are GRANT_ALL, MAX_DOUBLE": "",\n"SWICHAROO_MINSIZE_ACCEPT_ALL": 65536,\n"SWICHAROO_FILE_MAXSIZE_KB": 32768\n}\n');


if len(sys.argv) == 6 and sys.argv[1] == '--test_add_node':  # second arg is path to public key, third is nodename, 4,5 is ip, port
    # copy key from directory and insert as known node
    file = sys.argv[2]
    nodename = sys.argv[3]
    ip = sys.argv[4]
    port = sys.argv[5]
    print('Load up public key and insert it into our database:')
    with open(file, 'r') as f:
        pub_hex = f.read()
    # pub_key = RSA.importKey(bytes.fromhex(pub_hex))

    # insert into database
    dq.insert_node(ip, port, pub_hex)
else:
    setup()






























